@@ -16,7 +16,6 @@ replace it with C{cairocffi} as the two are API-compatible.
 """
 
 
-from io import StringIO
 from warnings import warn
 
 import os
@@ -38,15 +37,12 @@ from igraph.utils import _is_running_in_ipython, named_temporary_file
 
 __all__ = ("BoundingBox", "DefaultGraphDrawer", "Plot", "Point", "Rectangle", "plot")
 
-__license__ = "GPL"
-
 cairo = find_cairo()
-matplotlib, plt = find_matplotlib()
 
 #####################################################################
 
 
-class Plot(object):
+class Plot:
     """Class representing an arbitrary plot
 
     Every plot has an associated surface object where the plotting is done. The
@@ -89,8 +85,6 @@ class Plot(object):
     added by the L{Plot.add} method and removed by the L{Plot.remove} method.
     """
 
-    # pylint: disable-msg=E1101
-    # E1101: Module 'cairo' has no 'foo' member - of course it has! :)
     def __init__(self, target=None, bbox=None, palette=None, background=None):
         """Creates a new plot.
 
@@ -222,7 +216,7 @@ class Plot(object):
         """Sets the background color of the plot. C{None} means a
         transparent background. You can use any color specification here
         that is understood by the C{get} method of the current palette
-        or by L{igraph.colors.color_name_to_rgb}.
+        or by L{igraph.drawing.colors.color_name_to_rgb}.
         """
         if color is None:
             self._background = None
@@ -260,8 +254,6 @@ class Plot(object):
         """Marks the plot as dirty (should be redrawn)"""
         self._is_dirty = True
 
-    # pylint: disable-msg=W0142
-    # W0142: used * or ** magic
     def redraw(self, context=None):
         """Redraws the plot"""
         ctx = context or self._ctx
@@ -318,7 +310,16 @@ class Plot(object):
         self._surface.finish()
 
     def show(self):
-        """Saves the plot to a temporary file and shows it."""
+        """Saves the plot to a temporary file and shows it.
+
+        This method is deprecated from python-igraph 0.9.1 and will be removed in
+        version 0.10.0.
+
+        @deprecated: Opening an image viewer with a temporary file never worked
+            reliably across platforms.
+        """
+        warn("Plot.show() is deprecated from python-igraph 0.9.1", DeprecationWarning)
+
         if not isinstance(self._surface, cairo.ImageSurface):
             sur = cairo.ImageSurface(
                 cairo.FORMAT_ARGB32, int(self.bbox.width), int(self.bbox.height)
@@ -340,7 +341,7 @@ class Plot(object):
                 # should only happen on unknown platforms.
                 plat = platform.system()
                 raise NotImplementedError(
-                    "showing plots is not implemented " + "on this platform: %s" % plat
+                    "showing plots is not implemented on this platform: %s" % plat
                 )
             else:
                 os.system("%s %s" % (imgviewer, tmpfile))
@@ -368,12 +369,7 @@ class Plot(object):
         context.show_page()
         surface.finish()
         # Return the raw SVG representation
-        result = io.getvalue()
-        if hasattr(result, "encode"):
-            result = result.encode("utf-8")  # for Python 2.x
-        else:
-            result = result.decode("utf-8")  # for Python 3.x
-
+        result = io.getvalue().decode("utf-8")
         return result, {"isolated": True}  # put it inside an iframe
 
     @property
@@ -415,16 +411,23 @@ def plot(obj, target=None, bbox=(0, 0, 600, 600), *args, **kwds):
     @param target: the target where the object should be plotted. It can be one
       of the following types:
 
-        - C{None} -- an appropriate surface will be created and the object will
-          be plotted there.
+        - C{matplotib.axes.Axes} -- a matplotlib/pyplot axes in which the
+          graph will be plotted. Drawing is delegated to the chosen matplotlib
+          backend, and you can use interactive backends and matplotlib
+          functions to save to file as well.
+
+        - C{string} -- a file with the given name will be created and an
+          appropriate Cairo surface will be attached to it. The supported image
+          formats are: PNG, PDF, SVG and PostScript.
 
         - C{cairo.Surface} -- the given Cairo surface will be used. This can
           refer to a PNG image, an arbitrary window, an SVG file, anything that
           Cairo can handle.
 
-        - C{string} -- a file with the given name will be created and an
-          appropriate Cairo surface will be attached to it. The supported image
-          formats are: PNG, PDF, SVG and PostScript.
+        - C{None} -- a temporary file will be created and the object will be
+          plotted there. igraph will attempt to open an image viewer and show
+          the temporary file. This feature is deprecated from python-igraph
+          version 0.9.1 and will be removed in 0.10.0.
 
     @param bbox: the bounding box of the plot. It must be a tuple with either
       two or four integers, or a L{BoundingBox} object. If this is a tuple
@@ -455,7 +458,7 @@ def plot(obj, target=None, bbox=(0, 0, 600, 600), *args, **kwds):
       is 20 on each side.
 
     @keyword inline: whether to try and show the plot object inline in the
-      current IPython notebook. Passing ``None`` here or omitting this keyword
+      current IPython notebook. Passing C{None} here or omitting this keyword
       argument will look up the preferred behaviour from the
       C{shell.ipython.inlining.Plot} configuration key.  Note that this keyword
       argument has an effect only if igraph is run inside IPython and C{target}
@@ -465,6 +468,8 @@ def plot(obj, target=None, bbox=(0, 0, 600, 600), *args, **kwds):
 
     @see: Graph.__plot__
     """
+    _, plt = find_matplotlib()
+
     if hasattr(plt, "Axes") and isinstance(target, plt.Axes):
         result = MatplotlibGraphDrawer(ax=target)
         result.draw(obj, *args, **kwds)
